import { Component, OnInit } from '@angular/core';
import { NgbActiveModal } from '@ng-bootstrap/ng-bootstrap';
import { JhiAlertService } from 'ng-jhipster';
import { GitlabGroup } from 'app/shared/model/gitlab-group.model';
import { GitlabService } from 'app/shared/gitlab/gitlab.service';

@Component({
  selector: 'jhi-gitlab-path-selector',
  templateUrl: './gitlab-path-selector.component.html',
  styleUrls: ['./gitlab-path-selector.component.scss'],
})
export class GitlabPathSelectorComponent implements OnInit {
  groups: GitlabGroup[] | undefined;
  subgroups: GitlabGroup[] | undefined;

  isLoading = false;

  constructor(private activeModal: NgbActiveModal, private jhiAlertService: JhiAlertService, private gitlabService: GitlabService) {}

  ngOnInit(): void {
    this.isLoading = true;
    this.gitlabService.getGroups().subscribe(
      data => {
        this.groups = data;
        this.isLoading = false;
      },
      error => {
        this.jhiAlertService.error(error.error);
        this.isLoading = false;
      }
    );
  }

  /**
   * Utility function used to retrieve the upper groups in the
   * hierarchy
   */
  getParentGroups() {
    return this.groups?.filter(group => !group.parentId);
  }

  /**
   * Utility function used to retrieve the subgroups of
   * a given group
   * @param group to search for subGroups
   */
  getSubGroupsOfGroup(group: GitlabGroup) {
    return this.groups?.filter(subGroup => subGroup.parentId === group.id);
  }

  closeModal(group: GitlabGroup | undefined) {
    this.activeModal.close(group);
  }
}
